// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <iostream>

#include <functional>

#include <anna/core/core.hpp>

#include <anna/xml/Node.hpp>
#include <anna/xml/Compiler.hpp>

#include <anna/app/Application.hpp>

#include <anna/core/util/Encoder.hpp>

class Test : public app::Application {
public:
   Test ();
   
   void initialize () throw (RuntimeException);
   
private:
   void  run () throw (RuntimeException);      
};

using namespace std;

int main (int argc, const char** argv)
{
   CommandLine& commandLine (CommandLine::instantiate ());
   Test test;

   try {
      commandLine.initialize (argv, argc);
      commandLine.verify ();

      test.start ();
   }
   catch (Exception& ex) {
      cout << ex.asString () << endl;
   }
   
   return 0;
}

Test::Test () : 
   app::Application ("GenerateLogOn", "Database login encrypt keys generator", "1.0.0")
{
   CommandLine& commandLine (CommandLine::instantiate ());
      
   commandLine.add ("user", CommandLine::Argument::Mandatory, "User name"); 
   commandLine.add ("pwd", CommandLine::Argument::Mandatory, "Password");
   commandLine.add ("v", CommandLine::Argument::Optional, "Show original key in output file", false);
}

void Test::initialize () 
   throw (RuntimeException)
{
   Encoder::initialize ();
}

void Test::run () 
   throw (RuntimeException)
{
   CommandLine& commandLine (CommandLine::instantiate ());

   string user (commandLine.getValue ("user"));
   string password (commandLine.getValue ("pwd"));
   const bool verbose = commandLine.exists ("v");

   Encoder encoder;
   xml::Node root ("Logon");

   const EncodedData& euser = encoder.encode (user);

   xml::Node* node = root.createChild ("User");

   if (verbose)
      node->createAttribute ("PlainText", user);

   euser.asXML (node);

   const EncodedData& epassword = encoder.encode (password);

   node = root.createChild ("Password");

   if (verbose)
      node->createAttribute ("PlainText", password);

   epassword.asXML (node);

   xml::Compiler xmlCompiler;

   cerr << xmlCompiler.apply (&root) << endl << endl;
}

