// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <iostream>

#include <anna/core/mt/ThreadManager.hpp>
#include <anna/core/util/CommandLine.hpp>
#include <anna/core/mt/Mutex.hpp>
#include <anna/core/tracing/Logger.hpp>
#include <anna/core/mt/Runnable.hpp>
#include <anna/core/mt/Thread.hpp>
#include <anna/core/tracing/TraceWriter.hpp>

using namespace std;
using namespace anna;

class MyRunnable : public Runnable {
public:
   MyRunnable () : Runnable () {;}

private:
   void initialize () throw (RuntimeException);
   void run () throw (RuntimeException);
   void terminate () throw ();

   anna_complete_runnable (MyRunnable);
};

int main (const int argc, const char** argv)
{
   CommandLine& ccll = CommandLine::instantiate ();
   ThreadManager* threadManager (NULL);

   try {
      ccll.add ("maxthr", CommandLine::Argument::Mandatory, "Maximum number of threads");
      ccll.add ("mode", CommandLine::Argument::Mandatory, "Manager instance mode");
      ccll.add ("nthr", CommandLine::Argument::Mandatory, "Number of threads to create");

      ccll.initialize (argv, argc);
      ccll.verify ();

      Logger::initialize ("thread_manager", new TraceWriter ("file.trace", 2048000));

      ThreadManager::Mode::_v mode = ThreadManager::Mode::asEnumEx (ccll.getValue ("mode"));

      const int maxThread = ccll.getIntegerValue ("maxthr");
      int nthr = max (2, ccll.getIntegerValue ("nthr"));
     
      threadManager = new ThreadManager ("MyManager", mode, maxThread, Thread::Flag::Joinable);
      MyRunnable* myRunnable;
      Thread* thread;

      // Se genrean memory-leaks en los MyRunnable, pero no nos interesa complicar el ejemplo
      for (int ii = 0; ii < nthr; ii ++) {
         myRunnable = new MyRunnable ();
         thread = threadManager->createThread ();
         thread->start (*myRunnable);
      }
   
      // Espera la terminacin de todos los threads asociados al gestor que siguen corriendo
      threadManager->join ();

      // Crea un thread y espera su terminacin
      Thread* aux = threadManager->createThread ();
      myRunnable = new MyRunnable ();
      aux->start (*myRunnable);
      aux->join (); 
   }
   catch (RuntimeException& ex) {
      cout << ex.asString () << endl << endl;
      ex.trace ();
   }
}

void MyRunnable::initialize () 
   throw (RuntimeException)
{
   LOGDEBUG (
      string msg (asString ());
      msg += " | initialize ...";
      Logger::debug (msg, ANNA_FILE_LOCATION);
   );
}

void MyRunnable::run () 
   throw (RuntimeException)
{
   const int delay = rand () % 2000;

   LOGDEBUG (
      string msg (asString ());
      msg += functions::asText (" | Waiting: ", delay);
      msg += " ms";
      Logger::debug (msg, ANNA_FILE_LOCATION);
   );

   anna::functions::sleep ((Millisecond)delay);
}

void MyRunnable::terminate ()
   throw ()
{
   LOGDEBUG (
      string msg (asString ());
      msg += " | terminate ... ";
      Logger::debug (msg, ANNA_FILE_LOCATION);
   );
}

