// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_comm_DatagramSocket_hpp
#define anna_comm_DatagramSocket_hpp

#include <anna/comm/ClientSocket.hpp>

namespace anna {

namespace comm {

/**
   Esta clase implementa un Socket no orientado a conexion.
*/
class DatagramSocket : public ClientSocket {
public:
  /**
   * Indica el uso que tendr este DatagramSocket, si para recibir o para enviar.
   */
  enum Mode { ReadOnly, WriteOnly };

  /**
     Crear un Socket no orientado a conexion de un slo sentido.

     Los datagram socket definidos como \em Mode \em ReadOnly tienen que ser asociados al
     comunicador mediante la invocacin al mtodo anna::comm::Communicator::attach.

     Los datagram socket definidos como \em Mode \em WriteOnly tienen que invocar a al
     mtodo #connect antes de poder ser usados.

     \param mode Uso que se dar a este DatagramSocket.
     \param address Direccin en la que enviar o recibir mensajes.
     \param transportFactory Factoria de protocolos de transporte a usar por este sockets.
     \warning La factoria de protocolos debe estar disponible mientras el Socket esta activo.
  */
  DatagramSocket(const Mode mode, const INetAddress& address, TransportFactory* transportFactory = NULL);

  /**
     Devuelve el modo en que fue instanciado este Socket.
     \return el modo en que fue instanciado este Socket.
  */
  Mode getMode() const throw() { return a_mode; }

  /**
     Devuelve \em true si el socket fue instancia como slo-lectura.
     \return \em true si el socket fue instancia como slo-lectura.
  */
  const bool isReadOnly() const throw() { return a_mode == ReadOnly; }

  /**
     Devuelve \em true si el socket fue instancia como slo-lectura.
     \return \em true si el socket fue instancia como slo-lectura.
  */
  const bool isWriteOnly() const throw() { return a_mode == WriteOnly; }

  /**
     Prepara el socket para enviar a la direccion indicada en el constructor.
  */
  void connect() throw(RuntimeException);

private:
  const Mode a_mode;

  void do_write(const DataBlock&) throw(RuntimeException);
  int do_read(const char* data, const int maxSize) throw(RuntimeException);
};

}
}


#endif



