// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_comm_internal_Poll_hpp
#define anna_comm_internal_Poll_hpp

#include <unistd.h>
#include <sys/time.h>
#include <limits.h>

#include <anna/config/defines.hpp>
#include <anna/core/util/defines.hpp>
#include <anna/core/RuntimeException.hpp>
#include <anna/core/functions.hpp>

namespace anna {

namespace comm {

/**
   Clase que asegura que el comm::Reader no se quedara esperando indefinidamente a
   que un determinado mensaje haya sido tratado y mantiene la integridad de forma
   ya que asegura que el tratamiento de un mensaje solo tendra un unico signal enviado
    hacia el comm::Reader.
*/
class Poll {
public:
  Poll() : a_ptrTimeout(NULL), a_minfd(INT_MAX), a_maxfd(0) {
    FD_ZERO(&a_fdmask);
    FD_ZERO(&a_fdset);
  }

  void setTimeout(const Millisecond &timeout) throw();

  void waitMessage() throw(RuntimeException);
  int fetch() throw();
  bool isEmpty() const throw() { return a_pollr <= 0; }
  void clear() throw() { a_maxfd = 0; a_minfd = INT_MAX; FD_ZERO(&a_fdmask); FD_ZERO(&a_fdset);}
  void insert(const int fd) throw();
  void erase(const int fd) throw();

private:
  fd_set a_fdmask;
  fd_set a_fdset;
  timeval* a_ptrTimeout;
  int a_minfd;
  int a_maxfd;
  int a_pollr;
  int a_ifd;
  timeval a_timeout;

  static int select(const int maxfd, fd_set* fdset, timeval* timeout)
  throw(RuntimeException) {
    int result;

    do {
      result = ::select(maxfd + 1, fdset, NULL, NULL, timeout);
    } while(result == -1 && errno == EINTR);

    if(result == -1) {
      int xerrno = errno;
      throw RuntimeException("Error en anna::comm::Poll::select", xerrno, ANNA_FILE_LOCATION);
    }

    return result;
  }

  static int min(const Millisecond &n1, const Millisecond &n2) throw() { return (n1 < n2) ? n1 : n2; }
};

}
}

#endif
