// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_core_util_String_hpp
#define anna_core_util_String_hpp

#include <string>

#include <stdio.h>

#include <anna/core/util/defines.hpp>

namespace anna {

class DataBlock;

/**
 * Clase String que optimiza el uso y la conversi�n de tipos usados habitualmente.
 */
class String : public std::string {
public:
  /**
   * Indicadores que permiten ajustar el funcionamiento de la clase anna::String.
   *
   * \see anna::String
   */
  struct Flag { enum _v { None, ShowNull }; };

  /**
   * Constructor.
   * \param flag Indicadores que permiten ajustar el funcionamiento de la clase anna::String.
   */
  String(const Flag::_v flag = Flag::None) : a_flags(flag) {;}

  /**
   * Constructor.
   * \param str Cadena con la que iniciar el contenido de esta instancia. Puede ser NULL.
   * \param flag Indicadores que permiten ajustar el funcionamiento de la clase anna::String.
   */
  explicit String(const char* str, const Flag::_v flag = Flag::None) :
    std::string((str == NULL) ? ((flag & Flag::ShowNull) ? "<null>" : "") : str),
    a_flags(flag)
  {;}

  /**
   * Constructor copia.
   * \param other Instancia con la iniciar el contenido de esta instancia.
   */
  String(const String &other) : std::string(other), a_flags(other.a_flags) {;}


  /**
   * Constructor copia.
   * \param other Instancia con la iniciar el contenido de esta instancia.
   * \param flag Indicadores que permiten ajustar el funcionamiento de la clase anna::String.
   */
  explicit String(const std::string& other, const Flag::_v flag = Flag::None) : std::string(other), a_flags(flag) {;}

  /**
   * Convierte a may�sculas el contenido de esta instancia.
   */
  void toUpper() throw();

  /**
   * Convierte a min�sculas el contenido de esta instancia.
   */
  void toLower() throw();

  String& operator = (const char* vv) throw() { std::string::clear(); return operator<< (vv); }
  String& operator = (const int vv) throw() { std::string::clear(); return operator<< (vv); }
  String& operator = (const unsigned int vv) throw() { std::string::clear(); return operator<< (vv); }
  String& operator = (const bool vv) throw() { std::string::clear(); return operator<< (vv); }
  String& operator = (const S64 vv) throw() { std::string::clear(); return operator<< (vv); }
  String& operator = (const U64 vv) throw() { std::string::clear(); return operator<< (vv); }
  String& operator = (const float vv) throw() { std::string::clear(); return operator<< (vv); }
  String& operator = (const double vv) throw() { std::string::clear(); return operator<< (vv); }
  String& operator = (const std::string& vv) throw() { std::string::operator= (vv); return *this; }
  String& operator = (const DataBlock& vv) throw() { std::string::clear(); return operator<< (vv); }

  String& operator += (const char* vv) throw() { return operator<< (vv); }
  String& operator += (const int vv) throw() { return operator<< (vv); }
  String& operator += (const unsigned int vv) throw() { return operator<< (vv); }
  String& operator += (const bool vv) throw() { return operator<< (vv); }
  String& operator += (const S64 vv) throw() { return operator<< (vv); }
  String& operator += (const U64 vv) throw() { return operator<< (vv); }
  String& operator += (const float vv) throw() { return operator<< (vv); }
  String& operator += (const double vv) throw() { return operator<< (vv); }
  String& operator += (const std::string& vv) throw() { return *this << vv; }
  String& operator += (const DataBlock& vv) throw() { return *this << vv; }

  String& operator << (const char* vv) throw();
  String& operator << (const int vv) throw();
  String& operator << (const unsigned int vv) throw();
  String& operator << (const bool vv) throw() { std::string::operator+= ((vv == true) ? "true" : "false"); return *this; }
  String& operator << (const S64 vv) throw();
  String& operator << (const U64 vv) throw();
  String& operator << (const float vv) throw();
  String& operator << (const double vv) throw();
  String& operator << (const std::string& vv) throw() { std::string::operator+= (vv); return *this; }
  String& operator << (const DataBlock& vv) throw();

  /**
   * Formatea el contenido del \em float con la cadena recibida como par�metro.
   * La cadena de forma se interpreta de la misma forma que en el m�todo \em 'sprintf'
   * \param vv Valor a interpretar.
   * \param format Cadena usada para interpretar el valor (ver sprintf).
   * \return La cadena con el contenido interpretado.
   */
  static String format(const float vv, const char* format) throw() { return __format(vv, format); }

  /**
   * Formatea el contenido del \em double con la cadena recibida como par�metro.
   * La cadena de forma se interpreta de la misma forma que en el m�todo \em 'sprintf'
   * \param vv Valor a interpretar.
   * \param format Cadena usada para interpretar el valor (ver sprintf).
   * \return La cadena con el contenido interpretado.
   */
  static String format(const double vv, const char* format) throw() { return __format(vv, format); }

  /**
   * Formatea el contenido del DataBlock con el n�mero de caracteres por l�nea recibida como par�metro.
   * \param vv Valor a interpretar.
   * \param characterByLine N�mero de caracters por l�nea.
   * \return La cadena con el contenido interpretado.
   */
  static String format(const DataBlock& vv, const int characterByLine) throw();

  /**
   * Formatea el valor del par�metro como un n�mero en hexadecimal.
   * \param vv Valor a interpretar.
   * \return La cadena con el contenido interpretado.
   */
  static String hex(const int vv) { return __format(vv, "0x%x"); }

  /**
   * Formatea el valor del par�metro como un n�mero en hexadecimal.
   * \param vv Valor a interpretar.
   * \return La cadena con el contenido interpretado.
   */
  static String hex(const unsigned int vv) { return __format(vv, "0x%x"); }

  /**
   * Formatea el valor del par�metro como un n�mero en hexadecimal.
   * \param vv Valor a interpretar.
   * \return La cadena con el contenido interpretado.
   */
  static String hex(const S64 vv) {
    return __format(vv, "0x%llx");
    /*#ifdef __anna64__
       return __format (vv, "0x%lx");
    #else
       return __format (vv, "0x%llx");
    #endif
    */
  }

private:
  Flag::_v a_flags;

  template <typename T> static String __format(const T vv, const char* format) {
    char aux [64];
    sprintf(aux, format, vv);
    return String(aux);
  }
};

}

#endif
