// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_http_parser_Abstract_hpp
#define anna_http_parser_Abstract_hpp

#include <anna/config/defines.hpp>
#include <anna/core/define.autoenum.hpp>
#include <anna/core/RuntimeException.hpp>

namespace anna {

class DataBlock;

namespace http {

class Transport;
class Token;
class Message;

namespace parser {

class Abstract {
public:
  struct ClassType {
    enum _v { None = -1, WaitMessage, ReadHeader, WaitEndOfHeader, WaitChunkSize, ReadChunkSize, ReadChunkData, ReadChunkTrailers };
    anna_declare_enum(ClassType)
  };

protected:
  Abstract(const ClassType::_v classType) : a_classType(classType) {;}

  static void setState(Transport&, const ClassType::_v) throw();
  static void setLastChunkedByte(Transport&, const int lastChunkedByte) throw();
  static const DataBlock& getFullMessage(Transport&) throw();
  static void appendExtraParameter(Message*, const std::string& extraParameter) throw();

  virtual int processLine(Transport&, const DataBlock&, const Token& line) const throw(RuntimeException) = 0;

private:
  const ClassType::_v a_classType;

  ClassType::_v getClassType() const throw() { return a_classType; }
  std::string asString() const throw();

  friend class http::Transport;
};

}
}
}

#endif

