// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_test_Statistic_hpp
#define anna_test_Statistic_hpp

#include <anna/core/util/Average.hpp>
#include <anna/core/functions.hpp>
#include <anna/core/mt/Guard.hpp>
#include <anna/core/mt/Mutex.hpp>
#include <anna/core/RuntimeException.hpp>

namespace test {

using namespace anna;

class Statistic : public anna::Mutex {
public:
   Statistic () :
      a_messageCounter (0),
      a_successCounter (0),
      a_initTime (0),
      a_avgDelay ("AvgDelay"),
      a_errorCounter (0)
   {}

   void initTime () throw (RuntimeException) {
      if (a_initTime == 0) {
         Guard guard (this, "Statistic::initTime");
         if (a_initTime == 0)
            a_initTime = anna::functions::millisecond ();
      }
   }

   Millisecond getInitTime () const throw () { return a_initTime; }
   int getMessageCounter () const throw () { return a_messageCounter; }
   int getSuccessCounter () const throw () { return a_successCounter; }

   void countMessage () throw () { a_messageCounter ++; }
   void countSuccess () throw () { a_successCounter ++; }
   int countError () throw (RuntimeException) {
      Guard guard (this, "Statistic::countError");
      return ++ a_errorCounter;
   }
   void resetError () throw (RuntimeException) {
      if (a_errorCounter > 0) {
         Guard guard (this, "Statistic::countError");
         a_errorCounter = 0;
      }
   }

   void countDelay (const Millisecond value) throw (RuntimeException) {
      Guard guard (this, "Statistic::countDelay");
      a_avgDelay += value;
   }

   const Average <Microsecond>& getAvgDelay () const throw () { return a_avgDelay; }

   int getMessage () const throw () { return a_messageCounter; }

private:
   Millisecond a_initTime;
   int a_messageCounter;
   int a_successCounter;
   Average <Microsecond> a_avgDelay;
   int a_errorCounter;

};

}

#endif

