// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_timex_Clock_hpp
#define anna_timex_Clock_hpp

#include <anna/timex/TimeEvent.hpp>

namespace anna {

namespace timex {

/**
   Clase base para los relojes. Se activa de forma periodica cada #getTimeout milisegundos.

   @see Timer
*/
class Clock : public TimeEvent {
public:
  /**
     Constructor

     @param name Nombre logico de este reloj.
     @param timeout Periodo de activacion indicado en milisegundos.
  */
  Clock(const char* name, const Millisecond & timeout);

  /**
     Devuelve una cadena con la informacion referente a este reloj.
     \return Una cadena con la informacion referenta a este reloj.
  */
  std::string asString() const
  throw() {
    std::string msg("Clock { ");
    msg += TimeEvent::asString();
    msg += " | Name: ";
    msg += a_name;
    return msg += " }";
  }

  /**
     Metodo a reimplementar para establecer las acciones que se ejecutaran cada vez
     que expire el tiempo asociado al reloj.

     \return \em true si el reloj desea continuar recibiendo los pulsos periodicamente o
     \em false en otro caso.
  */
  virtual bool tick() throw(RuntimeException) = 0;

private:
  std::string a_name;

  void expire(Engine* timeController) throw(RuntimeException);
};

}
}

#endif


