// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <algorithm>

#include <anna/core/tracing/Logger.hpp>
#include <anna/core/tracing/TraceMethod.hpp>

#include <anna/xml/Node.hpp>
#include <anna/xml/Attribute.hpp>

#include <anna/app/functions.hpp>

#include <anna/comm/Network.hpp>
#include <anna/comm/Host.hpp>
#include <anna/comm/Server.hpp>

#include <anna/comm/Communicator.hpp>
#include <anna/comm/RoundRobinDelivery.hpp>
#include <anna/comm/functions.hpp>

using namespace std;
using namespace anna;

void comm::RoundRobinDelivery::do_initialize()
throw(RuntimeException) {
  LOGMETHOD(TraceMethod tm("comm::RoundRobinDelivery", "do_initialize", ANNA_FILE_LOCATION));
  a_iiserver = comm::Delivery::begin();
}

comm::Resource* comm::RoundRobinDelivery::do_apply()
throw(RuntimeException) {
  LOGMETHOD(TraceMethod tm(Logger::Local7, "comm::RoundRobinDelivery", "do_apply", ANNA_FILE_LOCATION));
  comm::Resource* result = NULL;

  if(comm::Delivery::size() == 0)
    return NULL;

  iterator end = a_iiserver;
  comm::Resource* w;

  do {
    w = resource(a_iiserver);

    if(w->isAvailable() == true && w->isEnabled() == true) {
      advance();
      result = w;
      break;
    }
  } while(advance() != end);

  return result;
}

comm::RoundRobinDelivery::iterator comm::RoundRobinDelivery::advance()
throw() {
  a_iiserver ++;

  if(a_iiserver == comm::Delivery::end())
    a_iiserver = comm::Delivery::begin();

  return a_iiserver;
}

string comm::RoundRobinDelivery::asString() const
throw() {
  string result = className();
  result += " { ";
  result += comm::Service::asString();
  return result += " }";
}

xml::Node* comm::RoundRobinDelivery::asXML(xml::Node* parent) const
throw() {
  xml::Node* node = parent->createChild("comm.RoundRobinDelivery");
  comm::Service::asXML(node);
  return node;
}

