// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


// Local
#include <anna/core/tracing/Configuration.hpp>

// STL
#include <string>


using namespace anna;
//using namespace anna::tracing;


//******************************************************************************
//---------------------------------------------------------------- Configuration
//******************************************************************************


//------------------------------------------------------------------------------
//---------------------------------------- Configuration::activateTraceTrigger()
//------------------------------------------------------------------------------
void anna::tracing::Configuration::activateTraceTrigger(const std::string & trigger, bool accumulate, anna::Logger::Level level) throw() {
  if(!accumulate) resetTraceTriggers();

  RegularExpression regEx(trigger); // not yet compiled (perhaps no regular expressions used)
  a_traceTriggers[regEx] = level;
}


//------------------------------------------------------------------------------
//-------------------------------------- Configuration::deactivateTraceTrigger()
//------------------------------------------------------------------------------
void anna::tracing::Configuration::deactivateTraceTrigger(const std::string & trigger) throw() {
  RegularExpression regEx(trigger); // not yet compiled (perhaps no regular expressions used)
  trace_trigger_map_t::iterator it = a_traceTriggers.find(regEx);

  if(it != a_traceTriggers.end()) a_traceTriggers.erase(it);
}


//------------------------------------------------------------------------------
//---------------------------------------------------- Configuration::asString()
//------------------------------------------------------------------------------
std::string anna::tracing::Configuration::asString(void) const throw() {
  std::string result = "";
// Gets selective tracing trigger map representation with this format:
//       'trigger1(desiredLevel1)|trigger2(desiredLevel2)| ... |triggerN(desiredLevelN)'

  if(a_traceTriggers.size() != 0) {
    trace_trigger_map_t::const_iterator it;

    for(it = a_traceTriggers.begin(); it != a_traceTriggers.end(); ++it) {
      result += (*it).first.getPattern();
      result += "(";
      result += Logger::asString((*it).second);
      result += ")|";
    }

    // Delete the last pipe: starts at 'size()-1', take 1 character:
    result.erase(result.size() - 1, 1);
  } else {
    result = "Currently, no trigger strings have been added for selective tracing";
  }

  result += "; Interpreted as Regular Expression: ";
  result += (a_readTraceTriggersAsRegexp) ? "yes" : "no";
  return (result);
}

