// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <string.h>
#include <limits.h>
#include <dirent.h>
#include <sys/types.h>
#include <stdio.h>

#include <algorithm>

#include <anna/config/defines.hpp>

#include <anna/io/Directory.hpp>
#include <anna/io/internal/sccs.hpp>

using namespace std;
using namespace anna;

io::Directory::Directory() :
  a_hasPattern(false) {
  io::sccs::activate();
}

void io::Directory::setPattern(const char* expression)
throw(RuntimeException) {
  if(a_hasPattern == true) {
    regfree(&a_regex);
    a_hasPattern = false;
  }

  if(expression == NULL)
    return;

  int ret;

  if((ret = regcomp(&a_regex, expression, REG_EXTENDED)) != 0) {
    char err[256];
    string msg("io::Directory | Pattern: ");
    msg += expression;
    msg += " | Error: ";

    if(regerror(ret, &a_regex, err, sizeof(err)))
      msg += err;
    else
      msg += "Invalid pattern";

    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  a_hasPattern = true;
}

void io::Directory:: read(const char* dirName, const Mode::_v mode)
throw(RuntimeException) {
  DIR* handle;
  dirent* entry;
  string file;

  if((handle = opendir(dirName)) == NULL) {
    const int aux_errno(errno);
    throw RuntimeException(string(dirName), aux_errno, ANNA_FILE_LOCATION);
  }

  a_files.clear();

  while((entry = readdir(handle)) != NULL) {
    if(!anna_strcmp(entry->d_name, ".") || !anna_strcmp(entry->d_name, ".."))
      continue;

    if(a_hasPattern == true)
      if(regexec(&a_regex, entry->d_name, 0, NULL, 0) != 0)
        continue;

    if(mode == Mode::FullPath) {
      file = dirName;
      file += "/";
      file += entry->d_name;
    } else
      file = entry->d_name;

    a_files.push_back(file);
  }

  closedir(handle);
}

io::Directory::const_iterator io::Directory::find(const std::string& file) const
throw() {
  return std::find(begin(), end(), file);
}

