// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


// Standard
#include <algorithm>

// Project
#include <anna/testing/TestDiameterCondition.hpp>

#include <anna/xml/Node.hpp>
#include <anna/xml/Compiler.hpp>
#include <anna/diameter/defines.hpp>
#include <anna/diameter/codec/functions.hpp>
#include <anna/diameter/codec/Message.hpp>
#include <anna/diameter/helpers/base/functions.hpp>

#include <anna/diameter/helpers/dcca/defines.hpp>
#include <anna/diameter/helpers/dcca/functions.hpp>
#include <anna/core/util/defines.hpp>
#include <anna/core/util/RegularExpression.hpp>


using namespace anna::testing;


const char* TestDiameterCondition::asText(const Type::_v type)
throw() {
  static const char* text [] = { "RegexpXml", "RegexpHex", "Fields" };
  return text [type];
}

bool TestDiameterCondition::exists() const throw() {
  if (a_type != Type::Fields) return (getRegexp() != "");

  return (a_code != "" || a_bitR != "" || a_hopByHop != "" || a_applicationId != "" || a_sessionId != "" || a_resultCode != "" || a_msisdn != "" || a_imsi != "" || a_serviceContextId != "");
}

bool TestDiameterCondition::comply(const anna::DataBlock &message) const throw() {

  if (a_type == Type::RegexpXml) {
    anna::diameter::codec::Message codecMsg;
    try { codecMsg.decode(message); } catch (anna::RuntimeException &ex) { ex.trace(); }

    //return codecMsg.isLike(getRegexp());
    // We will remove LF from both sides to ease regexp management:
    std::string regexp = getRegexp();
    regexp.erase(std::remove(regexp.begin(), regexp.end(), '\n'), regexp.end());
    anna::RegularExpression re(regexp);

    std::string msgString = codecMsg.asXMLString();
    msgString.erase(std::remove(msgString.begin(), msgString.end(), '\n'), msgString.end());

    return re.isLike(msgString);
  }
  else if (a_type == Type::RegexpHex) {
    anna::RegularExpression re(getRegexp());
    return re.isLike(anna::functions::asHexString(message));
  }

  // Basic
  std::string compare;
  anna::diameter::CommandId cid;

  if (a_code != "" || a_bitR != "") {
    try {
      cid = anna::diameter::codec::functions::getCommandId(message);
    }
    catch (anna::RuntimeException &) { return false; }
  }

  if (a_code != "") {
    compare = anna::functions::asString(cid.first);
    if (a_code != compare) return false;
  }

  if (a_bitR != "") {
    compare = (cid.second ? "1":"0");
    if (a_bitR != compare) return false;
  }

  if (a_hopByHop != "") {
    try {
      anna::diameter::HopByHop h = anna::diameter::codec::functions::getHopByHop(message);
      compare = anna::functions::asString(h);
    }
    catch (anna::RuntimeException &) { return false; }
    if (a_hopByHop != compare) return false;
  }

  if (a_applicationId != "") {
    try {
      anna::diameter::ApplicationId a = anna::diameter::codec::functions::getApplicationId(message);
      compare = anna::functions::asString(a);
    }
    catch (anna::RuntimeException &) { return false; }
    if (a_applicationId != compare) return false;
  }

  //if (matchSessionId) {
  if (a_sessionId != "") {
    try {
      compare = anna::diameter::helpers::base::functions::getSessionId(message);
    }
    catch (anna::RuntimeException &) { return false; }
    if (a_sessionId != compare) return false;
  }
  //}

  if (a_resultCode != "") {
    try {
      anna::U32 rc = anna::diameter::helpers::base::functions::getResultCode(message);
      compare = anna::functions::asString(rc);
    }
    catch (anna::RuntimeException &) { return false; }
    if (a_resultCode != compare) return false;
  }

  if (a_msisdn != "") {
    try {
      compare = anna::diameter::helpers::dcca::functions::getSubscriptionIdData(message, anna::diameter::helpers::dcca::AVPVALUES__Subscription_Id_Type::END_USER_E164);
    }
    catch (anna::RuntimeException &) { return false; }
    if (a_msisdn != compare) return false;
  }

  if (a_imsi != "") {
    try {
      compare = anna::diameter::helpers::dcca::functions::getSubscriptionIdData(message, anna::diameter::helpers::dcca::AVPVALUES__Subscription_Id_Type::END_USER_IMSI);
    }
    catch (anna::RuntimeException &) { return false; }
    if (a_imsi != compare) return false;
  }

  if (a_serviceContextId != "") {
    try {
      compare = anna::diameter::helpers::dcca::functions::getServiceContextId(message);
    }
    catch (anna::RuntimeException &) { return false; }
    if (a_serviceContextId != compare) return false;
  }

  return true;
}

anna::xml::Node* TestDiameterCondition::asXML(anna::xml::Node* parent) const
throw() {
  anna::xml::Node* result = parent->createChild("TestDiameterCondition");
  if (!exists()) return result;

  if (a_type == Type::RegexpXml) {
    if (getRegexp() != "") result->createAttribute("RegexpXml", getRegexp());
  }
  else if (a_type == Type::RegexpHex) {
    if (getRegexp() != "") result->createAttribute("RegexpHex", getRegexp());
  }
  else {
    if (a_code != "") result->createAttribute("Code", atoi(a_code.c_str()));
    if (a_bitR != "") result->createAttribute("BitR", ((a_bitR == "1") ? "yes":"no"));
    if (a_hopByHop != "") result->createAttribute("HopByHop", atoll(a_hopByHop.c_str()));
    if (a_applicationId != "") result->createAttribute("ApplicationId", atoll(a_applicationId.c_str()));
    if (a_sessionId != "") result->createAttribute("SessionId", a_sessionId);
    if (a_resultCode != "") result->createAttribute("ResultCode", atoi(a_resultCode.c_str()));
    if (a_msisdn != "") result->createAttribute("Msisdn", a_msisdn);
    if (a_imsi != "") result->createAttribute("Imsi", a_imsi);
    if (a_serviceContextId != "") result->createAttribute("ServiceContextId", a_serviceContextId);
  }

  result->createAttribute("ExpectedSource", a_rcvFromEntity ? "entity":"client");

  return result;
}
